package compiler

import (
	"git.sharkk.net/Sharkk/Mako/parser"
	"git.sharkk.net/Sharkk/Mako/vm"
)

// Compiler converts AST to bytecode
func Compile(program *parser.Program) *vm.Bytecode {
	c := &compiler{
		constants:    []any{},
		instructions: []vm.Instruction{},
	}

	for _, stmt := range program.Statements {
		c.compileStatement(stmt)
	}

	return &vm.Bytecode{
		Constants:    c.constants,
		Instructions: c.instructions,
	}
}

type compiler struct {
	constants    []any
	instructions []vm.Instruction
}

func (c *compiler) compileStatement(stmt parser.Statement) {
	switch s := stmt.(type) {
	case *parser.VariableStatement:
		c.compileExpression(s.Value)
		nameIndex := c.addConstant(s.Name.Value)
		c.emit(vm.OpSetGlobal, nameIndex)
	case *parser.EchoStatement:
		c.compileExpression(s.Value)
		c.emit(vm.OpEcho, 0)
	}
}

func (c *compiler) compileExpression(expr parser.Expression) {
	switch e := expr.(type) {
	case *parser.StringLiteral:
		constIndex := c.addConstant(e.Value)
		c.emit(vm.OpConstant, constIndex)
	case *parser.NumberLiteral:
		constIndex := c.addConstant(e.Value)
		c.emit(vm.OpConstant, constIndex)
	case *parser.Identifier:
		nameIndex := c.addConstant(e.Value)
		c.emit(vm.OpGetGlobal, nameIndex)
	}
}

func (c *compiler) addConstant(value any) int {
	c.constants = append(c.constants, value)
	return len(c.constants) - 1
}

func (c *compiler) emit(op vm.Opcode, operand int) {
	instruction := vm.Instruction{
		Opcode:  op,
		Operand: operand,
	}
	c.instructions = append(c.instructions, instruction)
}
